#==============================================================================
# package      DE_EPAGES::Google::UI::Shop
# base         DE_EPAGES::Order::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Google::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use JSON;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

sub UnitySave {
    my $self = shift;
    my ($Servlet) = @_;

    my $hValues = $Servlet->form->form($Servlet->object, 'SaveGoogleSearch');
    my $GoogleSiteVerificationToken = $hValues->{'GoogleSiteVerificationToken'};

    if($GoogleSiteVerificationToken =~ /^google[a-f0-9]+\.html$/  || $GoogleSiteVerificationToken eq ""){
        $self->SUPER::Save($Servlet);  
    }
    else{
        $Servlet->form->executeFormError({
            'Name'       => 'GoogleSearchHTMLFile',
            'Reason'     => 'WrongFormat',
        });
    }
}

sub UnityMBOViewGoogleSearch {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewGoogleSearch', { 'Servlet' => $Servlet });

    return $self->SUPER::View($Servlet);
}

#========================================================================================
# function     UnityMBOViewGoogleGTM
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->UnityMBOViewGoogleGTM($Servlet);
#----------------------------------------------------------------------------------------
# description  Subroutine for the the viewaction 'UnityMBO-ViewGoogleGTM'
#               Replicates the GTM fields that can be managed from the ep-ui editor
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | object
#========================================================================================
sub UnityMBOViewGoogleGTM {
    my $self = shift;
    my ($Servlet) = @_;

    my $Site = $Servlet->object;

    #-- These were taken from the corresponding ep-ui code
    #-- epages-ui\packages\fusion-frontend\src\web\utils\googleTagManager.ts
    my $GTMHeadRegEx = '<script[\s\S]+gtm\.start[\s\S]+https:\/\/[\w.]+\/gtm\.js[\s\S]+(GTM-[A-Z0-9]{1,7})[\s\S]+<\/script>';
    my $GTMBodyRegEx = '<iframe\s.+https:\/\/.+id=GTM-[A-Z0-9]{1,7}[\s\S]+<\/iframe>';

    my $Form = $Servlet->form;
    my $SaveGTM = $Form->value('SaveGTM');

    if ( defined $SaveGTM && $SaveGTM eq '1') {
        #-- Remove those leading/trailing spaces, just like in the UI
        my $GTMScriptCode = $Form->value('GTMScriptCode');
        $GTMScriptCode =~ s/^\s+|\s+$//g ;
        my $GTMNoScriptCode = $Form->value('GTMNoScriptCode');
        $GTMNoScriptCode =~ s/^\s+|\s+$//g ;
        my $GTMIsActive = $Form->value('GTMIsActive') ? JSON::true : JSON::false;

        #-- Either you give me a valid piece of script and noscript code from Google, or you are wiping it out (empty string)
        #-- This is already validated at a template level using js, but of course, we should also validate it here, as the api
        #-- will just die otherwise, and we choose not to eval this
        if ( ($GTMScriptCode eq '' || $GTMScriptCode =~ m/$GTMHeadRegEx/) && ($GTMNoScriptCode eq '' || $GTMNoScriptCode =~ m/$GTMBodyRegEx/) ) {

            my $aLanguages = $Site->get('Languages');
            foreach my $hLang ( @$aLanguages ) {
                $Site->sendUnityRequest('PATCH', '/shop', {'locale' => $hLang->{'DefaultLocaleID'}}, {
                    'userSettings' => {
                        'googleTagManager' => {
                            'headHtml' => $GTMScriptCode,
                            'bodyHtml' => $GTMNoScriptCode,
                            'active'   => $GTMIsActive,
                        }
                    }
                });
            }
        }
    }

    #-- Retrieve the currently stored gtm details from the ui
    #-- We don't care about the locale, the information is language independent
    my $hShopContent;
    eval {
        $hShopContent = $Site->sendUnityRequest('GET', '/shop');
    };
    if ( $@ ) {
        $hShopContent = undef;
    } else {
        $Servlet->vars('UI_GTM_CONTENTS', $hShopContent->{'userSettings'}->{'googleTagManager'});
    }

    $Servlet->vars('GTM_HEAD_SNIPPET_RE', $GTMHeadRegEx);
    $Servlet->vars('GTM_BODY_SNIPPET_RE', $GTMBodyRegEx);
}

sub UnitySFViewGoogleHTML {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object->get('Site');
    my $Response = $Servlet->response;
    $Response->header( 'Content-Type' => 'text/html' );

    my $GoogleSiteVerificationToken = $Shop->get('GoogleSiteVerificationToken');

    my $RequestURI;
    if ($self->_getServerVariables($Servlet, 'QUERY_STRING') =~ /^URI=([^&]+)/) {
        $RequestURI = $1;
    }

    if($GoogleSiteVerificationToken && $GoogleSiteVerificationToken eq $RequestURI) {
        # display html file for Google search
        my $Content= 'google-site-verification: ' . $GoogleSiteVerificationToken;
        $Response->content( $Content);
    } else {
        # display 404 error
        $Response->content('No google*.html available.');
        $Response->code('404');
    }

    $Servlet->vars('ExitAfterEvent', 1);

    return;
}

sub _getServerVariables {
    my $self = shift;
    my ($Servlet, $VariableName) = @_;

    return $Servlet->serverVariables($VariableName)
}


1;
